@@ -58,7 +58,7 @@
 #include "SocketAddr.h"
 #include "PerfSocket.hpp"
 #include "Extractor.h"
-#include "delay.hpp"
+#include "delay.h"
 #include "util.h"
 #include "Locale.h"
 #ifdef HAVE_SCHED_SETSCHEDULER
@@ -119,7 +119,7 @@ Client::~Client() {
     DELETE_ARRAY( mBuf );
 } // end ~Client
 
-const double kSecs_to_usecs = 1e6; 
+// const double kSecs_to_usecs = 1e6; 
 const double kSecs_to_nsecs = 1e9; 
 const int    kBytes_to_Bits = 8; 
 
@@ -202,7 +202,7 @@ void Client::RunRateLimitedTCP ( void ) {
 	    reportstruct->errwrite=0;
 	    currLen = write( mSettings->mSock, mBuf, mSettings->mBufLen );
 	    if ( currLen < 0 ) {
-		reportstruct->errwrite=1; 
+		reportstruct->errwrite=1;
 		currLen = 0;
 		if (
 #ifdef WIN32
@@ -346,7 +346,7 @@ void Client::RunTCP( void ) {
 	reportstruct->errwrite=0;
         currLen = write( mSettings->mSock, mBuf, mSettings->mBufLen );
         if ( currLen < 0 ) {
-	    reportstruct->errwrite=1; 
+	    reportstruct->errwrite=1;
 	    currLen = 0;
 	    if (
 #ifdef WIN32
@@ -420,6 +420,7 @@ void Client::Run( void ) {
     double delay_target = 0; 
     double delay = 0; 
     double adjust = 0;
+    double delay_lower_bounds;
 
     char* readAt = mBuf;
 
@@ -428,23 +429,24 @@ void Client::Run( void ) {
     //  and after Client::InitiateServer as during thes
     //  default socket timeouts are preferred.
     {
-	double sosndtimer;
-	sosndtimer = 0;    
+	int sosndtimer = 0;
+	// sosndtimer units microseconds
 	if (mSettings->mInterval) {
-	    sosndtimer = mSettings->mInterval / 2;
+	    sosndtimer = (int) (mSettings->mInterval * 1000000) / 2;
 	} else if (isModeTime(mSettings)) {
-	    sosndtimer = (mSettings->mAmount / 100.0) / 2;
+	    sosndtimer = (mSettings->mAmount * 10000) / 2;
 	} 
-	if (sosndtimer) {
+        // units nanoseconds for delay bounds
+	delay_lower_bounds = (double) sosndtimer * -1e3;
+
+	if (sosndtimer > 0) {
 #ifdef WIN32
             // Windows SO_RCVTIMEO uses ms
-	    DWORD timeout = (sosndtimer) * 1e3;
+	    DWORD timeout = (double) sosndtimer / 1e3;
 #else
 	    struct timeval timeout;
-	    double intpart, fractpart;
-	    fractpart = modf(sosndtimer, &intpart);
-	    timeout.tv_sec = (int) (intpart);
-	    timeout.tv_usec = (int) (fractpart * 1e6);
+	    timeout.tv_sec = sosndtimer / 1000000;
+	    timeout.tv_usec = sosndtimer % 1000000;
 #endif
 	    if (setsockopt( mSettings->mSock, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout, sizeof(timeout)) < 0 ) {
 		WARN_errno( mSettings->mSock == SO_SNDTIMEO, "socket" );
@@ -580,6 +582,11 @@ void Client::Run( void ) {
 	    // that spans the life of the thread and constantly adjust.
 	    // A negative delay means the iperf app is behind.
 	    delay += adjust;
+	    // Don't let delay grow unbounded
+	    if (delay < delay_lower_bounds) {
+		delay = delay_target;
+	    }
+
         }
 
         // Read the next data block from 
@@ -594,6 +601,8 @@ void Client::Run( void ) {
         currLen = write( mSettings->mSock, mBuf, mSettings->mBufLen );
         if ( currLen < 0 ) {
 	    reportstruct->errwrite = 1; 
+	    reportstruct->packetID--;
+	    reportstruct->emptyreport=1; 
 	    currLen = 0;
 	    if (
 #ifdef WIN32
@@ -601,7 +610,8 @@ void Client::Run( void ) {
 		errno != WSAECONNREFUSED
 #else
 		errno != EAGAIN && errno != EWOULDBLOCK &&
-		errno != EINTR  && errno != ECONNREFUSED
+		errno != EINTR  && errno != ECONNREFUSED &&
+		errno != ENOBUFS
 #endif
 		) {
 	        WARN_errno( 1, "write" );
