@@ -49,6 +49,7 @@
  *
  * ________________________________________________________________ */
 
+#include <math.h>
 #include "headers.h"
 #include "Settings.hpp"
 #include "util.h"
@@ -166,9 +167,9 @@ void reporter_printstats( Transfer_Info *stats ) {
 	    printf( stats->mEnhanced ? report_bw_jitter_loss_suppress_enhanced_format : report_bw_jitter_loss_format , stats->transferID, 
 		    stats->startTime, stats->endTime, 
 		    buffer, &buffer[sizeof(buffer)/2],
-		    stats->jitter*1000.0, stats->cntError, 
+		    0.0, stats->cntError, 
 		    stats->cntDatagrams, 
-		    0,0,0,0,0,0);
+		    0.0,0.0,0.0,0.0,0.0,0.0);
 	}
 	if ( stats->cntOutofOrder > 0 ) {
 	    printf( report_outoforder,
@@ -275,26 +276,28 @@ void reporter_serverstats( Connection_Info *nused, Transfer_Info *stats ) {
  */
 void reporter_reportsettings( ReporterData *data ) {
     int win, win_requested;
+    int pid =  (int)  getpid();
 
     win = getsock_tcp_windowsize( data->info.transferID,
                   (data->mThreadMode == kMode_Listener ? 0 : 1) );
     win_requested = data->mTCPWin;
-
     printf( "%s", separator_line );
     if ( data->mThreadMode == kMode_Listener ) {
-        printf( server_port,
-                (isUDP( data ) ? "UDP" : "TCP"), 
-                data->mPort );
+        printf(isEnhanced(data) ? server_pid_port : server_port,
+                (isUDP( data ) ? "UDP" : "TCP"),
+                data->mPort, pid );
     } else {
-        printf( client_port,
+        printf(isEnhanced(data) ? client_pid_port : client_port,
                 data->mHost,
                 (isUDP( data ) ? "UDP" : "TCP"),
-                data->mPort );
+                data->mPort, pid);
     }
+
     if ( data->mLocalhost != NULL ) {
         printf( bind_address, data->mLocalhost );
-        if ( SockAddr_isMulticast( &data->connection.local ) ) {
-            printf( join_multicast, data->mLocalhost );
+	if ((data->mThreadMode != kMode_Client) && \
+	    SockAddr_isMulticast(&data->connection.local)) {
+              printf( join_multicast, data->mLocalhost );
         }
     }
 
