@@ -1,4 +1,4 @@
-/*--------------------------------------------------------------- 
+/*---------------------------------------------------------------
  * Copyright (c) 1999,2000,2001,2002,2003                              
  * The Board of Trustees of the University of Illinois            
  * All Rights Reserved.                                           
@@ -17,7 +17,7 @@
  * copyright notice, this list of conditions and 
  * the following disclaimers. 
  *
- *     
+ *
  * Redistributions in binary form must reproduce the above 
  * copyright notice, this list of conditions and the following 
  * disclaimers in the documentation and/or other materials 
@@ -49,6 +49,7 @@
  *
  * ________________________________________________________________ */
 
+#include <math.h>
 #include "headers.h"
 #include "Settings.hpp"
 #include "util.h"
@@ -329,6 +330,9 @@ ReportHeader* InitReport( thread_Settings *agent ) {
 #else
         // set start time
         gettimeofday( &(reporthdr->report.startTime), NULL );
+        // set next report time
+	reporthdr->report.nextTime = reporthdr->report.startTime;
+	TimeAdd( reporthdr->report.nextTime, reporthdr->report.intervalTime );
         /*
          * Process the report in this thread
          */
@@ -360,9 +364,9 @@ void ReportPacket( ReportHeader* agent, ReportStruct *packet ) {
             // item
             while ( index == 0 ) {
                 Condition_Signal( &ReportCond );
-                Condition_Lock( ReportCond );
+                Condition_Lock( ReportDoneCond );
                 Condition_Wait( &ReportDoneCond );
-                Condition_Unlock( ReportCond );
+                Condition_Unlock( ReportDoneCond );
                 index = agent->reporterindex;
             }
             agent->agentindex = 0;
@@ -370,9 +374,9 @@ void ReportPacket( ReportHeader* agent, ReportStruct *packet ) {
         // Need to make sure that reporter is not about to be "lapped"
         while ( index - 1 == agent->agentindex ) {
             Condition_Signal( &ReportCond );
-            Condition_Lock( ReportCond );
+            Condition_Lock( ReportDoneCond );
             Condition_Wait( &ReportDoneCond );
-            Condition_Unlock( ReportCond );
+            Condition_Unlock( ReportDoneCond );
             index = agent->reporterindex;
         }
 
@@ -626,9 +630,51 @@ again:
                     goto again;
             }
             Condition_Signal( &ReportDoneCond );
-            usleep(10000);
-        } else {
-            //Condition_Unlock ( ReportCond );
+	    /* 
+	     * Suspend the reporter thread for 10 milliseconds
+	     *
+	     * This allows the thread to receive client or server threads'
+	     * packet events in "aggregates."  This can reduce context 
+	     * switching allowing for better CPU utilization, 
+	     * which is very noticble on CPU constrained systems.
+	     * 
+	     * If the realtime flag is set, then don't invoke the 
+	     * suspend.  This should give better reporter timing on 
+	     * higher end systems, where a busy-loop thread can be 
+	     * scheduled without impacting other threads.
+	     *
+	     * Note that if the reporter thread is signficantly slower 
+	     * than the Client or Server (traffic) threads this suspend 
+	     * will still be called even though the traffic threads can be 
+	     * blocked on the shared memory being full.  Probably should 
+	     * detect those and avoid the suspend under such conditions.  
+	     * It's not a big deal though because the traffic threads under  
+	     * normal conditions are much slower than the reporter thread.
+	     * The exception is when there are things like sustained 
+	     * write errors. Hence, such an optimization is deferred 
+	     * for a later date.
+	     *
+	     * Also note, a possible better implementation is for the 
+	     * reporter thread to block on a traffic thread's signal
+	     * instead of a 10 ms suspend.  That implementation 
+	     * would have to be profiled against this one to make sure
+	     * it indeed gave better performance.  Again, deferred.
+	     *
+	     * Final note, usleep() is being deprecated for nanosleep(), 
+	     * so use nanosleep if available
+	     */
+	    if ( !isRealtime( thread ) ) {
+#ifdef HAVE_NANOSLEEP 
+		{
+		    struct timespec requested;
+		    requested.tv_sec  = 0;
+		    requested.tv_nsec = 10000000L;
+		    nanosleep(&requested, NULL);
+		}
+#else 
+		usleep(10000);
+#endif
+	    }
         }
     } while ( 1 );
 }
@@ -1038,12 +1084,16 @@ int reporter_condprintstats( ReporterData *stats, MultiHeader *multireport, int
 	    if ( isMultipleReport(stats) ) {
 		reporter_handle_multiple_reports( multireport, &stats->info, force );
 	    }
-	    // Reset stats used by SUM now that the SUM has been reported
+	    /*
+	     * Reset transfer stats now that both the individual and SUM reports
+	     * have completed
+	     */
+	    if (stats->info.mUDP) {
+		stats->info.IPGcnt = 0;
+		stats->info.IPGsum = 0;
+	    }
 	    if (stats->info.mEnhanced) {
-		if (stats->info.mUDP) {
-		    stats->info.IPGcnt = 0;
-		    stats->info.IPGsum = 0;
-		} else if (stats->info.mTCP == (char)kMode_Client) {
+		if (stats->info.mTCP == (char)kMode_Client) {
 		    stats->info.tcp.write.WriteCnt = 0;
 		    stats->info.tcp.write.WriteErr = 0;
 		} else if (stats->info.mTCP == (char)kMode_Server) {
